/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: _integers.c
 * $Revision: 1.3 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: _integers.c,v 1.3 2002/02/21 09:55:51 ndarnis Exp $
 */

#include <config.h>
#include <stdlib.h>
#include <string.h>

#include "gdsl.h"
#include "_integers.h"

#ifdef HAVE_MEM_DEBUG
#include "_gdsl_mem.h"
#endif

extern gdsl_element_t 
alloc_integer (void *integer)
{
  int *n = (int *) integer;
#ifdef HAVE_MEM_DEBUG
  int *value = (int *) _gdsl_mem_alloc (sizeof (int));
#else
  int *value = (int *) malloc (sizeof (int));
#endif
  memcpy (value, n, sizeof (int));
  return (gdsl_element_t) value;
}

extern void 
free_integer (gdsl_element_t e)
{
#ifdef HAVE_MEM_DEBUG
  _gdsl_mem_free (e);
#else
  free (e);
#endif
}

extern int
display_integer (gdsl_element_t e, void *d)
{
  int **n = (int **) e;
  printf ("%d ", *n);
  return 1;
}

extern void
print_integer (gdsl_element_t e, FILE *file, void *d)
{
  int **n = (int **) e;
  fprintf (file, "%d ", *n);
}

extern int
compare_integers (gdsl_element_t e1, void *e2)
{
  int *a = (int *) e1;
  int *b = (int *) e2;
  if (*a == *b) return 0;
  if (*a < *b) return -1;
  return 1;
}

extern char *
get_integer (gdsl_element_t e)
{
  int *n = (int *) e;
  char *s = malloc (3);
  snprintf (s, 3, "%2d", *n);
  return s;
}
