/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: _strings.c
 * $Revision: 1.3 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: _strings.c,v 1.3 2002/02/21 09:55:51 ndarnis Exp $
 */

#include <config.h>
#include <stdlib.h>
#include <string.h>

#include "gdsl.h"
#include "_strings.h"

extern gdsl_element_t
alloc_string (void *string)
{
#ifdef HAVE_MEM_DEBUG
  char *tmp = (char *) _gdsl_mem_alloc (1 + strlen ((char *) string));
#else
  char *tmp =  (char *) malloc (1 + strlen ((char *) string));
#endif
  strcpy (tmp, (char *) string);
  tmp[strlen(string)] = '\0';
  return (gdsl_element_t) tmp;
}

extern void
free_string (gdsl_element_t e)
{
#ifdef HAVE_MEM_DEBUG
  _gdsl_mem_free (e);
#else
  free (e);
#endif
}

extern gdsl_element_t
copy_string (gdsl_element_t e)
{
#ifdef HAVE_MEM_DEBUG
  char *s = (char *) _gdsl_mem_alloc (1 + strlen ((char *)e));
#else
  char *s = (char *) malloc (1 + strlen ((char *) e));
#endif
  strcpy (s, (char *) e);
  return (gdsl_element_t) s;
}

extern int
display_string (gdsl_element_t e, void *d)
{
  printf ("%s ", (char *) e);
  return 1;
}

extern void
print_string (gdsl_element_t e, FILE *file, void *d)
{
  fprintf (file, "%s ", (char*) e);
}

extern int
compare_strings (gdsl_element_t s1, void *s2)
{
  return strcmp ((char *) s1, (char *) s2);
}

extern char *
get_string (gdsl_element_t e)
{
  return (char *) e;
}
