/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: main_2darray2.c
 * $Revision: 1.4 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: main_2darray.c,v 1.4 2002/02/21 09:55:51 ndarnis Exp $
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gdsl.h"

#ifdef HAVE_MEM_DEBUG
#include "_gdsl_mem.h"
#endif

#define NB_LINES   4UL
#define NB_COLUMNS 3UL

static gdsl_element_t alloue_nombre (void *e);
static void detruis_nombre (gdsl_element_t e);
static void affiche_nombre (gdsl_element_t e, FILE *file, bool flag1, bool flag2, void *user_data);

int main (void)
{
  int i, j, k = 0;
  gdsl_2darray_t m = gdsl_2darray_create ("m1", NB_LINES, NB_COLUMNS, alloue_nombre, detruis_nombre);

  for (i = 0; i < NB_LINES; i++)
    {
      for (j = 0; j < NB_COLUMNS; j++)
	{
	  int n = ++k;
	  gdsl_2darray_set_content (m, i, j, (void *) &n);
	}
    }

  printf ("Array %s (%ld x %ld):\n", gdsl_2darray_get_name (m),
	  gdsl_2darray_get_rows_count (m), 
	  gdsl_2darray_get_columns_count (m));
  gdsl_2darray_write (m, affiche_nombre, stdout, NULL);

  gdsl_2darray_destroy (m);

#ifdef HAVE_MEM_DEBUG
  printf ("<MEMORY_STATUS: %ld/>\n", _gdsl_mem_get_allocated_memory ());
#endif

  return EXIT_SUCCESS;
}

static gdsl_element_t alloue_nombre (void *e)
{
#ifdef HAVE_MEM_DEBUG
  int *n = (int *) _gdsl_mem_alloc (sizeof (int));
#else
  int *n = (int *) malloc (sizeof (int));
#endif
  memcpy (n, e, sizeof (int));
  return (gdsl_element_t) n;
}

static void detruis_nombre (gdsl_element_t e)
{
#ifdef HAVE_MEM_DEBUG
  _gdsl_mem_free (e);
#else
  free (e);
#endif
}

static void affiche_nombre (gdsl_element_t e, FILE *file, bool flag1, bool flag2, void *user_data)
{
  int *n = (int *) e;

  fprintf (file, "%s%02ld%s", flag1 == TRUE ? "( " : " ", *n, flag2 == TRUE ? " )\n" : "");
}
