/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: main_bstree.c
 * $Revision: 1.4 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: main_bstree.c,v 1.4 2002/02/21 09:55:51 ndarnis Exp $
 */

#include <config.h>
#include <stdio.h> 
#include <string.h>
#include <stdlib.h>

#include "gdsl.h"

#ifdef HAVE_MEM_DEBUG
#include "_gdsl_mem.h"
#endif

#include "_strings.h"
#include "_integers.h"

#define PERM_NB 100

int main (void)
{
  int choice;
  char name[50];
  gdsl_bstree_t t = gdsl_bstree_create ("my tree", alloc_string, free_string, compare_strings);

  do
    {
      printf ("\t\tBSTREE - MENU\n\n");
      printf ("\t 1> Insert\n"); 
      printf ("\t 2> Remove\n");
      printf ("\t 3> Flush\n");
      printf ("\t 4> Root content\n");
      printf ("\t 5> Size\n");
      printf ("\t 6> Height\n");
      printf ("\t 7> Search\n");
      printf ("\t 8> Display\n");
      printf ("\t 9> XML display\n");
      printf ("\t10> Dump\n");
      printf ("\t11> Insertion of a random permutation\n");
      printf ("\t 0> Exit\n\n");
      printf ("\t\tYour choice: ");
      scanf ("%d", &choice);
      
      switch (choice)
	{
	case 1:
	  printf ("Enter a string: ");
	  scanf ("%s", name);
	  
	  if (gdsl_bstree_insert (t, (void *) name) == NULL)
	    printf ("'%s' is already into the tree\n", name);
	  break;

	case 2:
	  if (gdsl_bstree_is_empty (t))
	    printf ("Empty tree!\n");
	  else
	    {
	      printf ("Enter a string: ");
	      scanf ("%s", name);
	      
	      if (gdsl_bstree_destroy_by_value (t, (void *) name))
		printf ("String '%s' removed from the tree\n", name);
	      else
		printf ("String '%s' not found\n", name);
	    }
	  break;

	case 3:
	  gdsl_bstree_flush (t);
	  break;

	case 4:
	  if (gdsl_bstree_is_empty (t))
	    printf ("Empty tree!\n");
	  else
	    {
	      display_string (gdsl_bstree_get_root (t), NULL);
	      printf ("\n");
	    }
	  break;
	  
	case 5:
	  printf ("Tree's size: %lu\n", gdsl_bstree_get_size (t));
	  break;

	case 6:
	  printf ("Tree's height: %lu\n", gdsl_bstree_height (t));
	  break;

	case 7: 
	  printf ("Enter a string: ");
	  scanf ("%s", name);
	  
	  if (gdsl_bstree_search (t, NULL, (void *) name))
	    printf ("String '%s' found\n", name);
	  else
	    printf ("String '%s' not found\n", name);
	  break;
	  
	case 8:
	  if (gdsl_bstree_is_empty (t))
	    printf ("EMPTY TREE!\n");
	  else
	    {
	      printf ("Tree's content: ");
	      gdsl_bstree_write (t, print_string, stdout, NULL);
	      printf ("\n");
#ifdef HAVE_GFX_DEBUG
	      gdsl_bstree_gfx_display (t, get_string);
#endif
	    }
	  break;

	case 9:
	  gdsl_bstree_write_xml (t, print_string, stdout, NULL);
	  break;

	case 10:
	  gdsl_bstree_dump (t, print_string, stdout, NULL);
	  break;
	  
	case 11:
	  {
	    int i;
	    gdsl_perm_t p = gdsl_perm_create ("PERM", PERM_NB, NULL, NULL, NULL, NULL);
	    gdsl_bstree_t nt = gdsl_bstree_create ("INTEGERS", alloc_integer, free_integer, compare_integers);

	    gdsl_perm_randomize (p);

	    for (i = 1; i <= PERM_NB; i++)
	      {
		int n = gdsl_perm_get (p, i);
		gdsl_bstree_insert (nt, &n);
	      }

	    gdsl_perm_destroy (p);
	    printf ("Tree's height: %lu\n", gdsl_bstree_height (nt));
#ifdef HAVE_GFX_DEBUG
	    gdsl_bstree_gfx_display (nt, get_integer);
#endif

	    gdsl_bstree_destroy (nt);
	  }
	  break;
	}   
    } 
  while (choice != 0);

  gdsl_bstree_destroy (t);
 
#ifdef HAVE_MEM_DEBUG
  printf ("MEMORY STATUS: %ld\n", _gdsl_mem_get_allocated_memory ());
#endif

  return EXIT_SUCCESS;
}
