/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: main_hash.c
 * $Revision: 1.3 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: main_hash.c,v 1.3 2002/02/21 09:55:51 ndarnis Exp $
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "gdsl.h"

#ifdef HAVE_MEM_DEBUG
#include "_gdsl_mem.h"
#endif

#include "_strings.h"

int main (void)
{
  int choix;
  gdsl_hash_t ht = gdsl_hash_create ("h1", NULL, alloc_string, free_string, NULL, 701);

  do
    {
      printf ("\t\tMENU - hash\n\n");
      printf ("\t1> Insert\n");
      printf ("\t2> Remove\n");
      printf ("\t3> Display\n");
      printf ("\t4> Flush\n");
      printf ("\t5> Fill factor\n");
      printf ("\t6> Dump\n");
      printf ("\t7> XML display\n");
      printf ("\t0> EXIT\n\n");
      printf ("\t\tYour choice: ");
      scanf ("%d", &choix);

      switch (choix)
	{
	case 1:
	  {
	    char nom[100];
	    printf ("String: ");
	    scanf ("%100s", nom);
	    if (gdsl_hash_insert (ht, nom) == NULL)
	      printf ("ERROR: Insert failed!\n");
	  }
	  break;

	case 2:
	  {
	    char nom[100];
	    gdsl_element_t e;

	    printf ("String: ");
	    scanf ("%100s", nom);
	    e = gdsl_hash_remove (ht, (void *) nom);
	    if (e == NULL)
	      printf ("String '%s' doesn't exist\n", nom);
	    else
	      free_string (e);
	  }
	  break;

	case 3:
	  gdsl_hash_write (ht, print_string, stdout, NULL);
	  printf ("\n");
	  break;

	case 4:
	  gdsl_hash_flush (ht);
	  break;

	case 5:
	  printf ("Fill factor: %f\n", gdsl_hash_fill_factor (ht));
	  break;

	case 6:
	  gdsl_hash_dump (ht, print_string, stdout, NULL);
	  break;

	case 7:
	  gdsl_hash_write_xml (ht, print_string, stdout, NULL);
	  break;
	}
    } 
  while (choix != 0);

  gdsl_hash_destroy (ht);

#ifdef HAVE_MEM_DEBUG
  printf ("MEMORY STATUS: %ld\n", _gdsl_mem_get_allocated_memory ());
#endif

  return EXIT_SUCCESS;
}


