/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: main_list.c
 * $Revision: 1.4 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: main_list.c,v 1.4 2002/02/21 09:55:51 ndarnis Exp $
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gdsl.h"

#ifdef HAVE_MEM_DEBUG
#include "_gdsl_mem.h"
#endif

static void affiche_liste_chaines_fwd (gdsl_list_t l);
static void affiche_liste_chaines_bwd (gdsl_list_t l);
static int affiche_chaine (gdsl_element_t chaine, void *user_infos);
static void affiche_element (gdsl_element_t chaine, FILE *file, void *user_data);
static gdsl_element_t alloue_chaine (void *chaine);
static void detruis_chaine (gdsl_element_t chaine);
static int compare_chaines (gdsl_element_t chaine, void *value);

int main (void)
{
  int choix = 0;
  int done = 0;

  gdsl_list_t l = gdsl_list_create ("my list", alloue_chaine, detruis_chaine);

  do
    {
      printf ("\t\tMENU - gdsl_list\n\n");
      printf ("\t 1> Creer une cellule\n");
      printf ("\t 2> Supprimer la premiere cellule\n");
      printf ("\t 3> Supprimer la derniere cellule\n");
      printf ("\t 4> Supprimer une cellule\n");
      printf ("\t 5> Afficher la liste ->\n");
      printf ("\t 6> Afficher la liste <-\n");
      printf ("\t 7> Vider la liste\n");
      printf ("\t 8> Cardinal de la liste\n");
      printf ("\t 9> Dump de la liste\n");
      printf ("\t10> Affichage XML de la liste\n");
      printf ("\t11> Recherche d'une place\n");
      printf ("\t12> Recherche d'un element\n");
      printf ("\t13> Tri de la liste\n");
      printf ("\t14> Max de la liste\n");
      printf ("\t 0> QUITTER\n\n");
      printf ("\t\tVotre choix: ");
      scanf ("%d", &choix);

      switch (choix)
	{
	case 1:
	  {
	    char nom[100];

	    printf ("Nom: ");
	    scanf ("%s", nom);

	    do
	      {
		int choix;

		printf ("\t\tMENU INSERTION\n\n");
		printf ("\t1> Inserer la cellule en debut de liste\n");
		printf ("\t2> Inserer la cellule en fin de liste\n");
		printf ("\t3> Inserer la cellule apres une autre cellule\n");
		printf ("\t4> Inserer la cellule avant une autre cellule\n");
		printf ("\t5> Afficher la liste\n");
		printf ("\t0> RETURN TO MAIN MENU\n\n");
		printf ("\t\tYou choice: ");
		scanf ("%d", &choix );

		switch (choix)
		  {
		  case 1:
		    {
		      gdsl_list_insert_head (l, nom);
		      done = 1;
		    }
		    break;

		  case 2:
		    {
		      gdsl_list_insert_tail (l, nom);
		      done = 1;
		    }
		    break;

		  case 3:
		    if (gdsl_list_is_empty (l))
		      printf ("Empty list!\n");
		    else
		      {
			char Nom[100];
			
			printf ("Nom de la cellule apres laquelle vous voulez inserer: ");
			scanf ("%s", Nom);
			
			if (!gdsl_list_cursor_search_by_value (l, compare_chaines, Nom))
			  printf ("La cellule '%s' n'existe pas\n", Nom);
			else
			  {
			    gdsl_list_cursor_insert_after (l, nom);
			    done = 1;
			  }
		      }
		    break;

		  case 4:
		    if (gdsl_list_is_empty (l))
		      printf ("Empty list!\n");
		    else
		      {
			char Nom[100];
			
			printf ("Nom de la cellule avant laquelle vous voulez inserer: ");
			scanf ("%s", Nom);
			
			if (!gdsl_list_cursor_search_by_value (l, compare_chaines, Nom))
			  printf ("La cellule '%s' n'existe pas\n", Nom);
			else
			  {
			    gdsl_list_cursor_insert_before (l, nom);
			    done = 1;
			  }
		      }
		    break;

		  case 5:
		    if (gdsl_list_is_empty (l))
		      printf ("Empty list!\n");
		    else
		      affiche_liste_chaines_fwd (l);
		    break;

		  case 0:
		    done = 1;
		    break;
		  }
	      } 
	    while (!done);
	  }
	  break;

	case 2:
	  if (gdsl_list_is_empty (l))
	    printf ("Empty list!\n");
	  else
	    {
	      gdsl_list_destroy_head (l);
	      done = 1;
	    }
	  break;

	case 3:
	  if (gdsl_list_is_empty (l))
	    printf ("Empty list!\n");
	  else
	    {
	      gdsl_list_destroy_tail (l);
	      done = 1;
	    }
	  break;

	case 4:
	  {
	    char nom[100];

	    if (gdsl_list_is_empty (l))
	      printf ("Empty list!\n");
	    else
	      {
		printf ("Nom de la cellule  supprimer: ");
		scanf ("%s", nom);

		if (!gdsl_list_destroy_by_value (l, compare_chaines, nom))
		  printf ("La cellule '%s' n'existe pas!\n", nom);
		else
		  {
		    printf ("La cellule '%s' a ete supprimee!\n", nom);
		    done = 1;
		  }
	      }
	  }
	  break;

	case 5:
	  if (gdsl_list_is_empty (l))
	    printf ("Empty list!\n");
	  else
	    affiche_liste_chaines_fwd (l);
	  break;

	case 6:
	  if (gdsl_list_is_empty (l))
	    printf ("Empty list!\n");
	  else
	    affiche_liste_chaines_bwd (l);
	  break;
	  
	case 7:
	  if (gdsl_list_is_empty (l))
	    printf ("Empty list!\n");
	  else
	    gdsl_list_flush (l);
	  break;

	case 8:
	  printf ("Card( %s ) = %ld\n", gdsl_list_get_name (l), gdsl_list_get_card (l));
	  break;

	case 9:
	  if (!gdsl_list_is_empty (l))
	    gdsl_list_dump (l, NULL, stdout, NULL);
	  break;

	case 10:
	  if (!gdsl_list_is_empty (l))
	    gdsl_list_write_xml (l, affiche_element, stdout, NULL);
	  break;

	case 11:
	  {
	    ulong pos;
	    gdsl_element_t e;
	    printf ("Entrez la position de la place a rechercher: ");
	    scanf ("%uld", &pos);
	    e = gdsl_list_search_by_position (l, pos);
	    if (e != NULL)
	      affiche_element (e, stdout, NULL);
	  }
	  break;
	  
	case 12:
	  {
	    char nom[100];
	    gdsl_element_t e;

	    printf ("Nom de la cellule a rechercher: ");
	    scanf ("%s", nom);
	    
	    e = gdsl_list_search_by_value (l, compare_chaines, nom);
	    if (e == NULL)
	      printf ("La cellule '%s' n'existe pas!\n", nom);
	    else
	      {
		printf ("La cellule '%s' a ete trouvee: ", nom);
		affiche_element (e, stdout, NULL);
		printf ("\n");
	      }
	  }
	  break;

	case 13:
	  {
	    gdsl_element_t max = alloue_chaine ("ZZZZZZZZZZZZZZZZZZZZZZZZZZZ");
	    l = gdsl_list_merge_sort (l, compare_chaines, max);
	    detruis_chaine (max);
	  }
	  break;
	  
	case 14:
	  {
	    gdsl_element_t e = gdsl_list_search_max (l, compare_chaines);
	    printf ("Element max: %s\n", (char *) e);
	  }
	  break;

	case 15:
	  {
	    int i = 0;

	    for (gdsl_list_cursor_move_to_head (l); gdsl_list_cursor_get (l); gdsl_list_cursor_move_forward (l))
	      {
		
		char toto[50];
		sprintf (toto, "_%d", i++);
		gdsl_list_cursor_destroy_before (l);
		gdsl_list_cursor_insert_before (l, toto);
	      }
	    gdsl_list_cursor_destroy_before (l);
	  }
	  break;
	}
    } 
  while (choix != 0);

  gdsl_list_destroy (l);

#ifdef HAVE_MEM_DEBUG
  printf ("MEMORY STATUS: %ld\n", _gdsl_mem_get_allocated_memory ());
#endif

  return EXIT_SUCCESS;
}

static void 
affiche_liste_chaines_fwd (gdsl_list_t l)
{
  int i = 0;
  printf ("%s = ( ", gdsl_list_get_name (l));
  /*
  gdsl_list_map_forward( l, affiche_chaine, NULL);
  printf( ")\n" );
  
  printf( "%s = ( ", gdsl_list_get_name( l ) );
  for( gdsl_list_cursor_move_to_head( l ); gdsl_list_cursor_get( l ); gdsl_list_cursor_move_forward( l ) )
    {
      affiche_chaine( gdsl_list_cursor_get( l ), NULL );
    }
  */

    for (gdsl_list_cursor_move_to_head (l); i < gdsl_list_get_card (l); gdsl_list_cursor_move_forward (l))
    {
      affiche_chaine (gdsl_list_cursor_get (l), NULL);
      i++;
    }
  printf (")\n");
}

static void 
affiche_liste_chaines_bwd (gdsl_list_t l)
{
  gdsl_element_t e;

  /*
  printf ("%s = ( ", gdsl_list_get_name (l));
  gdsl_list_map_backward (l, affiche_chaine, NULL);
  printf (")\n");
  */
  printf ("%s = ( ", gdsl_list_get_name (l));
  for (gdsl_list_cursor_move_to_tail (l); (e = gdsl_list_cursor_get (l)); gdsl_list_cursor_move_backward (l))
    affiche_chaine (e, NULL);
  printf (")\n");
}

static int 
affiche_chaine (gdsl_element_t chaine, void *user_infos)
{
  printf ("%s, ", (char *) chaine);
  return 1;
}

static void
affiche_element (gdsl_element_t chaine, FILE *file, void *user_data)
{
  fprintf (file, "%s", (char *) chaine);
}

static gdsl_element_t 
alloue_chaine (void *chaine)
{
#ifdef HAVE_MEM_DEBUG
  char *string = (char *) _gdsl_mem_alloc (1 + strlen ((char *) chaine));
#else
  char *string = (char *) malloc (1 + strlen ((char *) chaine));
#endif
  strcpy (string, (char *) chaine);
  return (gdsl_element_t) string;
}

static void 
detruis_chaine (gdsl_element_t chaine)
{
#ifdef HAVE_MEM_DEBUG
  _gdsl_mem_free (chaine);
#else
  free (chaine);
#endif
}

static int 
compare_chaines (gdsl_element_t chaine, void *value)
{
  return strcmp ((char *) chaine, (char *) value); 
}
