/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: mainllbintree.c
 * $Revision: 1.3 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: main_llbintree.c,v 1.3 2002/02/21 09:55:51 ndarnis Exp $
 */

#include <config.h>
#include <stdio.h> 
#include <string.h>
#include <stdlib.h>

#include "gdsl.h"

#include "_strings.h"

static int my_display_string (_gdsl_bintree_t t, void *d);

int main (void)
{
  _gdsl_bintree_t g, d, t, t1, t2, copy;

  g = _gdsl_bintree_create( (gdsl_element_t)"b" );
  d = _gdsl_bintree_create( (gdsl_element_t)"f" );
  t1 = _gdsl_bintree_make( (gdsl_element_t)"d", g, d );
  g = _gdsl_bintree_create( (gdsl_element_t)"n" );
  d = _gdsl_bintree_create( (gdsl_element_t)"q" );
  t2 = _gdsl_bintree_make( (gdsl_element_t)"o", g, d );
  t = _gdsl_bintree_make( (gdsl_element_t)"k", t1, t2 );
  
  printf( "T:\n" );
  _gdsl_bintree_write_xml( t, print_string, stdout, NULL );

  copy = _gdsl_bintree_copy( t, copy_string );
  printf( "COPY OF T: \n" );
  _gdsl_bintree_dump( copy, print_string, stdout, NULL );

#ifdef HAVE_GFX_DEBUG
  _gdsl_bintree_gfx_display( copy, get_string );
#endif

  _gdsl_bintree_left_rot( &t );
#ifdef HAVE_GFX_DEBUG
  _gdsl_bintree_gfx_display( t, get_string );
#endif

  _gdsl_bintree_right_rot( &t );
#ifdef HAVE_GFX_DEBUG
  _gdsl_bintree_gfx_display( t, get_string );
#endif

  _gdsl_bintree_right_rot( &t );
#ifdef HAVE_GFX_DEBUG
  _gdsl_bintree_gfx_display( t, get_string );
#endif

  _gdsl_bintree_left_rot( &t );
  _gdsl_bintree_left_right_rot( &t );
#ifdef HAVE_GFX_DEBUG
  _gdsl_bintree_gfx_display( t, get_string );
#endif

  printf( "\nT in prefixed order: " );
  _gdsl_bintree_prefix_parse( t, my_display_string, NULL );
  printf( "\nT in infixed order: " );
  _gdsl_bintree_infix_parse( t, my_display_string, NULL );
  printf( "\nT in postfixed order: " );
  _gdsl_bintree_postfix_parse( t, my_display_string, NULL );

  printf( "\n\nCOPY OF T in prefixed order: " );
  _gdsl_bintree_prefix_parse( copy, my_display_string, NULL );
  printf( "\nCOPY OF T in infixed order: " );
  _gdsl_bintree_infix_parse( copy, my_display_string, NULL );
  printf( "\nCOPY OF T in postfixed order: " );
  _gdsl_bintree_postfix_parse( copy, my_display_string, NULL );
  printf( "\n\n" );

  _gdsl_bintree_destroy( copy, free_string );
  _gdsl_bintree_destroy( t, NULL );

#ifdef HAVE_MEM_DEBUG
  printf ("MEMORY STATUS: %ld\n", _gdsl_mem_get_allocated_memory ());
#endif

  return EXIT_SUCCESS;
}

static int my_display_string (_gdsl_bintree_t t, void *d)
{
  return display_string (_gdsl_bintree_get_content (t), d);
}
