/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: main_lllist.c
 * $Revision: 1.3 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: main_lllist.c,v 1.3 2002/02/21 09:55:51 ndarnis Exp $
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gdsl.h"
#include "_strings.h"

#ifdef HAVE_MEM_DEBUG
#include "_gdsl_mem.h"
#endif

static int display_node (_gdsl_list_node_t n, void *d);

int main (void)
{
  _gdsl_list_t l = _gdsl_list_create( alloc_string("a") );

  _gdsl_list_node_t n1 = _gdsl_list_node_alloc();
  _gdsl_list_node_t n2 = _gdsl_list_node_alloc();

  _gdsl_list_node_set_content(n1,alloc_string("c"));
  _gdsl_list_node_set_content(n2,alloc_string("b"));

  _gdsl_list_insert_after(n2,l);
  _gdsl_list_link(n2,n1);

  printf( "WRITE:\n" );
  _gdsl_list_write( l, print_string, stdout, NULL);

  printf( "\n\nDUMP:\n" );
  _gdsl_list_dump( l, print_string, stdout, NULL );

  printf( "\nWRITE XML:\n" );
  _gdsl_list_write_xml( l, print_string, stdout, NULL );

  printf( "\nMAP FORWARD:\n" );
  _gdsl_list_map_forward(l,display_node,NULL);
  printf( "\n" );

  _gdsl_list_destroy(l,free_string);

  exit( EXIT_SUCCESS );
}

static int display_node (_gdsl_list_node_t n, void *d)
{
  gdsl_element_t e = _gdsl_list_node_get_content (n);
  print_string (e, stdout, d);
  return 1;
}

