/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: main_queue.c
 * $Revision: 1.3 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: main_queue.c,v 1.3 2002/02/21 09:55:51 ndarnis Exp $
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gdsl.h"

#ifdef HAVE_MEM_DEBUG
#include "_gdsl_mem.h"
#endif

static void affiche_file (gdsl_queue_t f);
static int affiche_cellule (gdsl_element_t e, void *user_infos);
static void affiche_element (gdsl_element_t e, FILE *file, void *user_data);
static gdsl_element_t cree_element (void *nom);
static void detruis_element (gdsl_element_t e);

int main (void)
{
  int choix = 0;
  gdsl_queue_t f = gdsl_queue_create ("my queue", cree_element, detruis_element);
  
  do
    {
      printf ("\t\tMENU - gdsl_queue\n\n");
      printf ("\t1> Enfiler\n");
      printf ("\t2> Defiler\n");
      printf ("\t3> Head\n");
      printf ("\t4> Tail\n");
      printf ("\t5> Display\n");
      printf ("\t6> Flush\n");
      printf ("\t7> Dump\n");
      printf ("\t8> XML display\n");
      printf ("\t0> QUIT\n\n" );
      printf ("\t\tYour cooice: " );
      scanf ("%d", &choix );

      switch (choix)
	{
	case 1:
	  {
	    float value;
	    printf ("Value: ");
	    scanf ("%f", &value);
	    gdsl_queue_put (f, (void *) &value);
	  }
	  break;

	case 2:
	  if (!gdsl_queue_is_empty (f))
	    detruis_element (gdsl_queue_get (f));
	  else
	    printf ("Empty queue!\n");
	  break;
	  
	case 3:
	  {
	    float *sommet;
	    if (!gdsl_queue_is_empty (f)) 
	      {
		sommet = (float *) gdsl_queue_get_head (f);
		printf ("Head = %f\n", *sommet);
	      }
	    else
	      printf ("Empty queue!\n");
	  }
	  break;

	case 4:
	  {
	    float *sommet;
	    if (!gdsl_queue_is_empty (f)) 
	      {
		sommet = (float *) gdsl_queue_get_tail (f);
		printf ("Tail = %f\n", *sommet);
	      }
	    else
	      printf ("Empty queue!\n");
	  }
	  break;

	case 5:
	  if (gdsl_queue_is_empty (f))
	    printf ("La file '%s' est vide\n", gdsl_queue_get_name (f));
	  else
	    affiche_file (f);
	  break;

	case 6:
	  if (gdsl_queue_is_empty (f))
	    printf ("Empty queue!\n");
	  else
	    gdsl_queue_flush (f);
	  break;

	case 7:
	  if (!gdsl_queue_is_empty (f))
	    gdsl_queue_dump (f, NULL, stdout, NULL);
	  break;

	case 8:
	  gdsl_queue_write_xml (f, affiche_element, stdout, NULL);
	  break;
	}
    } 
  while (choix != 0);

  gdsl_queue_destroy (f);

#ifdef HAVE_MEM_DEBUG
  printf ("MEMORY STATUS: %ld\n", _gdsl_mem_get_allocated_memory ());
#endif

  return EXIT_SUCCESS;
}

static void 
affiche_file (gdsl_queue_t f)
{
  printf ("%s:\n", gdsl_queue_get_name (f));
  gdsl_queue_map (f, affiche_cellule, NULL);
  printf ("\n");
}

static int 
affiche_cellule (gdsl_element_t e, void *user_infos)
{
  float *f = (float *) e;
  printf ("%f\n", *f);
  return 1;
}

static void
affiche_element (gdsl_element_t e, FILE *file, void *user_data)
{
  float *f = (float *) e;
  fprintf (file, "%f", *f);
}

static gdsl_element_t 
cree_element (void *valeur)
{
#ifdef HAVE_MEM_DEBUG
  float *f = (float *) _gdsl_mem_alloc (sizeof (float));
#else
  float *f = (float *) malloc (sizeof (float));
#endif
  memcpy (f, valeur, sizeof (float));
  return (gdsl_element_t) f;
}

static void 
detruis_element (gdsl_element_t e)
{
#ifdef HAVE_MEM_DEBUG
  _gdsl_mem_free (e);
#else
  free (e);
#endif
}
