/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: main_stack.c
 * $Revision: 1.3 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: main_stack.c,v 1.3 2002/02/21 09:55:51 ndarnis Exp $
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gdsl.h"

#ifdef HAVE_MEM_DEBUG
#include "_gdsl_mem.h"
#endif

static void print_stack (gdsl_stack_t p);
static int print_cell (gdsl_element_t e, void *user_infos);
static void print_element (gdsl_element_t e, FILE *file, void *user_data);
static gdsl_element_t alloc_float (void *nom);
static void free_float (gdsl_element_t element);

int main (void)
{
  int choix = 0;
  gdsl_stack_t p = gdsl_stack_create ("my stack", alloc_float, free_float);

  do
    {
      printf ("\t\tMENU - gdsl_stack\n\n");
      printf ("\t1> Push\n");
      printf ("\t2> Pop\n");
      printf ("\t3> Get\n");
      printf ("\t4> Display\n");
      printf ("\t5> Flush\n");
      printf ("\t6> Dump\n");
      printf ("\t7> XML display\n");
      printf ("\t0> EXIT\n\n");
      printf ("\t\tYour choice: ");
      scanf ("%d", &choix);

      switch (choix)
	{
	case 1:
	  {
	    float value;

	    printf ("Float value: ");
	    scanf ("%f", &value);
	    gdsl_stack_push (p, (void *) &value);
	  }
	  break;

	case 2:
	  if (!gdsl_stack_is_empty (p))
	    free_float (gdsl_stack_pop (p));
	  else
	    printf ("Empty stack!\n");
	  break;
	  
	case 3:
	  {
	    float *sommet;

	    if (!gdsl_stack_is_empty (p)) 
	      {
		sommet = (float *) gdsl_stack_get_top (p);
		printf ("Value = %f\n", *sommet);
	      }
	    else
	      printf ("Empty stack!\n");
	  }
	  break;

	case 4:
	  if (gdsl_stack_is_empty (p))
	    printf ("Empty stack!\n");
	  else
	    print_stack (p);
	  break;

	case 5:
	  if (gdsl_stack_is_empty (p))
	    printf ("Empty stack!\n");
	  else
	    gdsl_stack_flush (p);
	  break;

	case 6:
	  gdsl_stack_dump (p, NULL, stdout, NULL);
	  break;

	case 7:
	  gdsl_stack_write_xml (p, print_element, stdout, NULL);
	  break;
	}
    } 
  while (choix != 0);

  gdsl_stack_destroy (p);

#ifdef HAVE_MEM_DEBUG
  printf ("MEMORY STATUS: %ld\n", _gdsl_mem_get_allocated_memory ());
#endif

  return EXIT_SUCCESS;
}

static void 
print_stack (gdsl_stack_t p)
{
  printf ("%s:\n", gdsl_stack_get_name (p));
  gdsl_stack_map (p, print_cell, NULL);
  printf ("\n");
}

static int 
print_cell (gdsl_element_t e, void *user_infos)
{
  float *f = (float *) e;
  printf ("%f\n", *f);
  return 1;
}

static void
print_element (gdsl_element_t e, FILE *file, void *user_data)
{
  float *f = (float *) e;
  fprintf (file, "%f", *f);
}

static gdsl_element_t 
alloc_float (void *valeur)
{
#ifdef HAVE_MEM_DEBUG
  float *f = (float *) _gdsl_mem_alloc (sizeof (float));
#else
  float *f = (float *) malloc (sizeof (float));
#endif
  memcpy (f, valeur, sizeof (float));
  return (gdsl_element_t) f;
}

static void 
free_float (gdsl_element_t e)
{
#ifdef HAVE_MEM_DEBUG
  _gdsl_mem_free (e);
#else
  free (e);
#endif
}
