/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: mem.c
 * $Revision: 1.3 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: mem.c,v 1.3 2002/02/21 09:55:51 ndarnis Exp $
 */

#include <config.h>
#include <stdlib.h>

#include "mem.h"

static void * (*malloc_fun) (size_t) = malloc;
static void * (*calloc_fun) (size_t, size_t) = calloc;
static void * (*realloc_fun) (void *, size_t) = realloc;
static void (*free_fun) (void *) = free;

/******************************************************************************/
/* Public functions                                                           */
/******************************************************************************/

extern void *
mem_alloc (size_t size)
{
  return (*malloc_fun) (size);
}

extern void *
mem_calloc (size_t number, size_t size)
{
  return (*calloc_fun) (number, size);
}

extern void *
mem_realloc (void *ptr, size_t size)
{
  return (*realloc_fun) (ptr, size);
}

extern void 
mem_free (void *p)
{
  (*free_fun) (p);
}

extern void 
mem_set_functions (void * (*malloc_user_fun) (size_t),
		   void (*free_user_fun) (void *),
		   void * (*calloc_user_fun) (size_t, size_t),
		   void * (*realloc_user_fun) (void *, size_t))
{
  malloc_fun = malloc_user_fun;
  calloc_fun = calloc_user_fun;
  realloc_fun = realloc_user_fun;
  free_fun = free_user_fun;
}

extern void 
mem_set_default_functions (void)
{
  malloc_fun = malloc;
  calloc_fun = calloc;
  realloc_fun = realloc;
  free_fun = free;
}
